
//********************************************************************
//*    PGMID.        APPLICATION PROGRAM DATA UNIT (APDU) HANDLING.  *
//*    AUTHOR.       BERND R. FIX   >Y<                              *
//*    DATE WRITTEN. 07/02/10.                                       *
//*    COPYRIGHT.    (C) BY BERND R. FIX. ALL RIGHTS RESERVED.       *
//*                  LICENSED MATERIAL - PROGRAM PROPERTY OF THE     *
//*                  AUTHOR. REFER TO COPYRIGHT INSTRUCTIONS.        *
//*    REMARKS.                                                      *
//********************************************************************

/*
 *  $Log: apdu.h,v $
 *  Revision 1.6  2007/02/17 16:56:32  y67
 */

#ifndef _APDU_H_
#define _APDU_H_

#include "hal.h"

//=====================================================================
//	APDU data structures
//=====================================================================

#define	CLA		0
#define	INS		1
#define	P0		2
#define	P1		3
#define	LC		4
#define	DATA	5

//=====================================================================
//	Status words
//=====================================================================

#define	SW_WRONG_LEN		0x6700
#define	SW_WRONG_DATA		0x6A80
#define	SW_NOT_SUPPORTED	0x6A81
#define	SW_WRONG_P1P2		0x6A86
#define	SW_WRONG_LE		0x6C00
#define	SW_WRONG_INS		0x6D00
#define	SW_WRONG_CLA		0x6E00
#define SW_OTHER		0x6F00
#define	SW_OK			0x9000

//=====================================================================
//	Function prototypes:
//=====================================================================

void	apduInit	(byte* atr);
bool	apduGetCommand	(byte* in);
int	apduGetAddr	(byte* in);
bool	apduGetData	(byte ins, byte* data, int num, bool single);
bool	apduResponse	(byte ins, byte* data, int num, word SW);
void	apduStatus	(word SW);

//=====================================================================
// Acknowledgments
//=====================================================================

static __inline__ void apduAcknowledge  (byte ins) { uartSendByte ( ins); }
static __inline__ void apduCAcknowledge (byte ins) { uartSendByte (~ins); }


#endif

