
//********************************************************************
//*    PGMID.        APDU HANDLING.                                  *
//*    AUTHOR.       BERND R. FIX   >Y<                              *
//*    DATE WRITTEN. 07/02/10.                                       *
//*    COPYRIGHT.    (C) BY BERND R. FIX. ALL RIGHTS RESERVED.       *
//*                  LICENSED MATERIAL - PROGRAM PROPERTY OF THE     *
//*                  AUTHOR. REFER TO COPYRIGHT INSTRUCTIONS.        *
//*    REMARKS.                                                      *
//********************************************************************

/*
 *  $Log: apdu.c,v $
 *  Revision 1.2  2007/02/17 16:53:47  y67
 */

///////////////////////////////////////////////////////////////////////
// import external declarations.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "apdu.h"

//=====================================================================
// Initialize and sent ATR
//=====================================================================

void apduInit (byte* atr) {

	// initialize serial communication
	bool isDirect = (atr[0] == 0x3B); 
	uartInit (isDirect);
	
	//-------------------------------------------------------------------
	// process ATR: Since we need to simulate a booting CardOS,
	// time spent here is welcome.
	//-------------------------------------------------------------------
	int numHist = atr[1] & 15;		// number of historical bytes
	int TxFlags = atr[1] >> 4;		// Tx definition flags
	
	// calculate number of bytes to start of hist
	// and the position of the TC byte
	int count = 2;
	int n, mask = 1;
	for (n = 0; n < 4; n++, mask <<= 1) {
		if ((TxFlags & mask) != 0) {
			count++;
		}
	}
	// send ATR
	uartSendBytes (atr, count + numHist);
}

//=====================================================================
// receive next command APDU
//=====================================================================
bool apduGetCommand (byte* in)
{
	// get the next five byte from input buffer
	return (uartReceiveBytes (in, 5) == 5);
}

//=====================================================================
// get address from command APDU
//=====================================================================
int apduGetAddr (byte* in)
{
	int v1 = (in[P0] & 0xFF);
	int v2 = (in[P1] & 0xFF);
	return (v1 << 8 | v2);
}

//=====================================================================
// get data for command
//=====================================================================
bool apduGetData (byte ins, byte* data, int num, bool single)
{
	if (single) {
		int n;
		for (n = 0; n < num; n++) {
			// send ~ACK
			apduCAcknowledge (ins);
			// receive next byte
			data[n] = uartReceiveByte ();
		}
		return TRUE;
	} else {
		// send ACK
		apduAcknowledge (ins);
		// receive next byte
		return uartReceiveBytes (data, num) == num;
	}
}

//=====================================================================
// send reponse APDU with data
//=====================================================================
bool apduResponse (byte ins, byte* data, int num, word SW)
{
	// send INS byte as ACK
	apduAcknowledge (ins);
	
	// send requested data
	uartSendBytes (data, num);
	
	// send status word.
	uartSendWord (SW);
	
	// report status
	return TRUE;
}

//=====================================================================
// send error status word
//=====================================================================
void apduStatus (word SW)
{
	// send status word.
	uartSendWord (SW);
}
