
//********************************************************************
//*    PGMID.        HANDLE SERIAL PORT COMMUNICATION.               *
//*    AUTHOR.       BERND R. FIX   >Y<                              *
//*    DATE WRITTEN. 07/02/10.                                       *
//*    COPYRIGHT.    (C) BY BERND R. FIX. ALL RIGHTS RESERVED.       *
//*                  LICENSED MATERIAL - PROGRAM PROPERTY OF THE     *
//*                  AUTHOR. REFER TO COPYRIGHT INSTRUCTIONS.        *
//*    REMARKS.                                                      *
//********************************************************************

/*
 *  $Log: uart.c,v $
 *  Revision 1.7  2007/04/25 22:42:00  y67
 */

///////////////////////////////////////////////////////////////////////
// import external declarations.

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include "hal.h"

#define	BIT(x)  (1<<(x))

///////////////////////////////////////////////////////////////////////
//	low-level I/O routines
///////////////////////////////////////////////////////////////////////

extern	void	initT0		(void);		// initialize serial comm.

extern	void	sendT0		(byte x);	// send byte (direct or
extern	void	isendT0		(byte x);	// indirect convention)

extern	byte	receiveT0	(void);		// receive byte (direct or
extern	byte	ireceiveT0	(void);		// indirect convention)

///////////////////////////////////////////////////////////////////////
// Delay processing
///////////////////////////////////////////////////////////////////////

static __inline__ void _delay_loop (uint16_t __count)
{
    __asm__ volatile (
        "1: sbiw %0,1" "\n\t"
        "brne 1b"
        : "=w" (__count)
        : "0" (__count)
    );
}

///////////////////////////////////////////////////////////////////////
//	Low-level receive/send routines.
///////////////////////////////////////////////////////////////////////

static bool useDirect = FALSE;

//=====================================================================
//	Initialize UART handling for direct or indirect convention
//=====================================================================
void uartInit (bool isDirect)
{
	useDirect = isDirect;
	initT0();
}

//=====================================================================
//	Delay processing for given number of millis
//=====================================================================
void uartDelay (int ms)
{
	_delay_loop ((ms * 279) / 4);
}

//=====================================================================
// Send number of bytes to serial port:
// This methods is not using buffered transmit, so the call
// blocks until all requested bytes are sent. 
//=====================================================================

void uartSendWord (word w)
{
	uartSendByte ((w >> 8) & 0xFF);
	uartSendByte ( w       & 0xFF);
}

void uartSendByte (byte b)
{
	if (useDirect)
		sendT0 (b);
	else
		isendT0 (b);
	return;
} 

void uartSendBytes (byte* data, int num)
{
	int n;
	for (n = 0; n < num; n++) {
 		// send byte
	    uartSendByte (data[n]);
	}
}

//=====================================================================
// Receive a number of bytes:
// Blocking call to receive a given number of bytes. The call will
// terminate with an error if no data is received within one second.
//=====================================================================

byte uartReceiveByte ()
{
	if (useDirect)
		return receiveT0 ();
	return ireceiveT0 ();
}

int uartReceiveBytes (byte* data, int num)
{
	int n;
	for (n = 0; n < num; n++) {
		// get data byte and advance read pointer.
		data[n] = uartReceiveByte ();
	}
	return num;
}
